<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Message;
use Teknisa\MAN\Utilities\Util;
use Teknisa\MAN\Services;
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\MAN\Services\MetaService;

class MetaController extends CrudWithParameterBag {
    
    protected $metaService;
    protected $parameterBag;
    protected $util;
    
    protected $dataSourceNameArray = array (
        'metaunidade'        => 'meta_unidade',
        'metames'            => 'meta_mes',
        'metamesDia'         => 'meta_mesDia',
        'metadia'            => 'meta_dia',
        'metamodalidade'     => 'meta_modalidade',
        'metamodalidadeData' => 'metamodalidadeData',
        'metadiaespecial'    => 'meta_diaespecial',
        'metahora'           => 'meta_hora',
        'metahoraData'       => 'meta_horaData'
    );
    
    protected $parameterBagColumnsArray = array (
        'metaunidade'        => array('NRORG', 'CDOPERADOR'),
        'metames'            => array('NRORG', 'CDFILIAL','CDANO', 'TCDANO'),
        'metamesDia'         => array('NRORG', 'CDFILIAL','CDANO', 'TCDANO'),
        'metadia'            => array('NRORG', 'CDFILIAL', 'NRDIA', 'CDMES', 'CDANO'),
        'metamodalidade'     => array('NRORG', 'CDFILIAL', 'NRDIA', 'CDMES', 'CDANO', 'IDMODALIDADE'),
        'metamodalidadeData' => array('NRORG', 'CDFILIAL', 'DATA', 'IDMODALIDADE'),
        'metadiaespecial'    => array('NRORG', 'CDFILIAL', 'DATAESP'),
        'metahora'           => array('NRORG', 'CDFILIAL', 'CDHORA', 'NRDIA', 'CDMES', 'CDANO'),
        'metahoraData'       => array('NRORG', 'CDFILIAL', 'HORA', 'DATA')
    );

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, $crudService, MetaService $metaService,
    Util $util) {
        $this->crudService  = $crudService;
        $this->metaService  = $metaService;
        $this->util = $util;
        parent::__construct($dataSourceManager, $parameterBag);
    }
    
    public function saveRow ($request, $response, $dataSourceName) {
        
        try {
        
            $dataSet = $request->getDataSet();
            $rows = $request->getDataSet()->getRows();
            $novasRows = array();
            if(!empty($rows)){
                foreach ($rows as $key => &$row) {
                    if (!isset($row['__group'])) {
                        $novasRows[] = $row;
                    }
                    
                }    
                $rows = $novasRows;
            }
            $newDataSet = new DataSet($dataSourceName, $rows );
            $persistedRows = $this->dataSourceManager->persist($newDataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function findMetaMes (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['metames'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['metames'];
        parent::find($request, $response);
    }
    
     public function findMetaMesDia (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['metamesDia'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['metamesDia'];
        parent::find($request, $response);
        
        $rows = $response->getDataSets()[0]->getRows();
        $acumulado = 0;
        $aux = '';
        foreach ($rows as &$row) {
            if($row['CDMES'] != $aux){
                $acumulado = 0;
            }
            $acumulado += $row['VRMETADIA'];
            $row['ACUMULADO'] = $acumulado;
            $aux = $row['CDMES'];
        }
        $response->addDataSet( new DataSet('meta_mesDia', $rows ));
        
    }
    
    public function saveMetaMes (DTO\Request\DataSet $request, DTO\Response $response) {
        $dataSourceName = $this->dataSourceNameArray['metames'];
        $rows = $request->getDataSet()->getRows();
        $novasRows = array();
        
        foreach ($rows as &$row) {
            if (gettype($row['VRMES']) == 'string' && isset($row['CDMES'])) {
                $row['VRMES'] = $this->util->moneyToDouble($row['VRMES']);
                $row['VRMES_inline'] = $this->util->moneyToDouble($row['VRMES']); 
                $row['VRSEM'] = $this->util->moneyToDouble($row['VRSEM']);
                $row['VRSEM_inline'] = $this->util->moneyToDouble($row['VRSEM']); 
                
                $novasRows[] = $row;
            }
        }
        $rows = $novasRows;
        $this->saveRow($request, $response, $dataSourceName);
    }
    
    public function deleteMetaMes (DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['metames'];
        try{
            $this->metaService->deleteRegistrosFilhos($this->util->getParamsAsArray($request));
            $this->deleteCrud($request,$response);
        }catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function findMetaDia (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['metadia'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['metadia'];
        parent::find($request, $response);
    }
    
    public function saveMetaDia (DTO\Request\DataSet $request, DTO\Response $response) {
        $dataSourceName = $this->dataSourceNameArray['metadia'];
        foreach ($request->getDataSet()->getRows() as &$row) {
            $row['VRTICKETDIA'] = (float)$row['VRTICKETDIA'];
            $row['VRTICKETDIA_inline'] = $row['VRTICKETDIA'];
            $row['changed_VRTICKETDIA'] = $row['VRTICKETDIA']; 
            $row['VRPERCDIA'] = (float)$row['VRPERCDIA'];
        }
        $this->saveRow($request, $response, $dataSourceName);
    }
    
    public function findMetaModalidade (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['metamodalidade'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['metamodalidade'];
        parent::find($request, $response);
    }
    
    public function findMetaModalidadeData (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['metamodalidadeData'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['metamodalidadeData'];
        parent::find($request, $response);
    }
    
    public function saveMetaModalidade (DTO\Request\DataSet $request, DTO\Response $response) {
        $dataSourceName = $this->dataSourceNameArray['metamodalidade'];
        
        foreach ($request->getDataSet()->getRows() as &$row) {
            $row['VRPERCMODALIDADE'] = (float)$row['VRPERCMODALIDADE'];
            $row['VRPERCMODALIDADE_inline'] = $row['VRPERCMODALIDADE']; 
        }
        
        $this->saveRow($request, $response, $dataSourceName);
    }
    
    public function findMetaDiaEspecial (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['metadiaespecial'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['metadiaespecial'];
        parent::find($request, $response);
    }
    
    public function saveMetaDiaEspecial (DTO\Request\DataSet $request, DTO\Response $response) {
        $dataSourceName = $this->dataSourceNameArray['metadiaespecial'];
        foreach ($request->getDataSet()->getRows() as $key =>  &$row) {
            $row['PERCCORRECAO'] = (float)$row['PERCCORRECAO'];
            $row['PERCCORRECAO_inline'] = $row['PERCCORRECAO'];
        }
        $this->saveRow($request, $response, $dataSourceName);
    }
    
    public function deleteMetaDiaEspecial (DTO\Request\Row $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['metadiaespecial'];
        try{
            $this->deleteCrud($request, $response);
        }catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function findMetaHora (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['metahora'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['metahora'];
        parent::find($request, $response);
    }
    
    public function findMetaHoraData (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['metahoraData'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['metahoraData'];
        parent::find($request, $response);
    }
    
    public function saveMetaHora (DTO\Request\DataSet $request, DTO\Response $response) {
        $dataSourceName = $this->dataSourceNameArray['metahora'];
        
        foreach ($request->getDataSet()->getRows() as &$row) {
            $row['VRPERCHORA'] = (float)$row['VRPERCHORA'];
            if(isset($row['VRPERCHORA_inline'])) {
                $row['VRPERCHORA_inline'] = $row['VRPERCHORA_inline']; 
            }
        }
        
        $this->saveRow($request, $response, $dataSourceName);
    }
      
    public function deleteCrud(DTO\Request $request, DTO\Response $response) {
		try {
			$request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $this->crudService->delete($response, $dataSet);
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	public function recalcularMeta (DTO\Request $request, DTO\Response $response) {
	    $row = $request->getRow();
	    if($row['TODASFILIAIS'] && empty($row['CDFILIAL'])) {
	       $this->metaService->recalculoMetaTodasFiliais($row['CDOPERADOR'], $row['NRORG']);
	    } else {
	        $this->metaService->recalculoMetaFilial($row['CDFILIAL'], $row['NRORG'], $row['CDOPERADOR']);
	    }
    }
    
     public function findUnidade (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['metaunidade'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['metaunidade'];
        parent::find($request, $response);
    }    
    
    public function estimarMeta (DTO\Request $request, DTO\Response $response) {
	    $row = $request->getRow();
	    is_array($row['CDFILIAL']) ? 0 : $row['CDFILIAL'] = array($row['CDFILIAL']);
	    $PERCENTUAL = (Float) str_replace(',', '.', $row['PERCENTUAL']);
	    
	    if(!isset($row['PERIODOMESIN'])){
	        $row['PERIODOMESIN'] = null;
	    }
	    
	    if(!isset($row['PERIODOMESFN'])){
	        $row['PERIODOMESFN'] = null;
	    }
	    
	    foreach ($row['CDFILIAL'] as $CDFILIAL) {
	        $this->metaService->estimaMetaFilial($CDFILIAL, $row['NRORG'], $PERCENTUAL, $row['PERIODO'], $row['DTINI'], $row['DTFIN'], $row['ANODATA'], $row['PERIODOMESIN'], $row['PERIODOMESFN']);
	    }
    }
    
    public function syncDashboardData (DTO\Request\Filter $request, DTO\Response $response) {
        $response = $this->metaService->syncDashboardData();
        return $response;
    }
    
    public function deleteMesCheckedRows (DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = $this->dataSourceNameArray['metames'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['metames'];
        foreach($this->util->getParamsAsArray($request) as $row){
            $this->metaService->deleteRegistrosFilhos($row);
        }
        parent::delete($request, $response);
    }
    
    public function deleteDiaEspecialCheckedRows (DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = $this->dataSourceNameArray['metadiaespecial'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['metadiaespecial'];
        parent::delete($request, $response);
    }
    
    
    public function getTipoMeta (DTO\Request $request, DTO\Response $response) {
	    $row = $request->getRow();
	    try {
	        $tipoMeta = $this->metaService->getTipoMeta($row['CDFILIAL']);
            $response->addDataSet(new DataSet('tipoMeta', $tipoMeta));
         } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
}